package pt.gov.dgarq.roda.core.data.adapter.sort;

import java.io.Serializable;

/**
 * @author Rui Castro
 */
public class SortParameter implements Serializable {
	private static final long serialVersionUID = 5682003551885541798L;

	private String name;
	private boolean descending = false;

	/**
	 * Constructs an empty {@link SortParameter}.
	 */
	public SortParameter() {
	}

	/**
	 * Constructs a {@link SortParameter} cloning an existing
	 * {@link SortParameter}.
	 * 
	 * @param sortParameter
	 *            the {@link SortParameter} to clone.
	 */
	public SortParameter(SortParameter sortParameter) {
		this(sortParameter.getName(), sortParameter.isDescending());
	}

	/**
	 * Constructs a {@link SortParameter} with the given parameters.
	 * 
	 * @param name
	 *            the name of the attribute to sort
	 * @param descending
	 *            descending or ascending order.
	 */
	public SortParameter(String name, boolean descending) {
		setName(name);
		setDescending(descending);
	}

	/**
	 * @see Object#toString()
	 */
	public String toString() {
		return "SortParameter(name=" + getName() + ", descending="
				+ isDescending() + ")";
	}

	/**
	 * @see Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		boolean equal = true;

		if (obj != null && obj instanceof SortParameter) {
			SortParameter other = (SortParameter) obj;
			equal = equal
					&& (getName() == other.getName() || getName().equals(
							other.getName()));
			equal = equal && (isDescending() == other.isDescending());
		} else {
			equal = false;
		}

		return equal;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the descending
	 */
	public boolean isDescending() {
		return descending;
	}

	/**
	 * @param descending
	 *            the descending to set
	 */
	public void setDescending(boolean descending) {
		this.descending = descending;
	}

}
